import collections

import six


class Block(object):
    def __init__(self):
        self._output = ''
        self._indent_level = 0

    def open_value(self, text):
        pass

    def close_value(self, text):
        pass

    def simple_value(self, text):
        pass


class Serializer(object):
    def __init__(self):
        self._indent_level = 0
        self._output = ''
        self._needs_indent = False

    def put(self, line):
        if self._needs_indent:
            self._output += '  ' * self._indent_level
            self._needs_indent = False
        self._output += line

    def eol(self):
        self._output += '\n'
        self._needs_indent = True

    def is_valid_pair_key(self, key):
        # TODO
        return isinstance(key, str)

    def serialize_int(self, data):
        self.put(str(data))
        self.eol()

    def serialize_float(self, data):
        self.put(str(data))
        self.eol()

    def serialize_string(self, data):
        self.put("'{}'".format(data))
        self.eol()

    def serialize_pair(self, key, val):
        self.put(key)
        self.put(': ')
        self.serialize_any(val)

    def serialize_map_inner(self, val):
        # TODO
        for key in sorted(val):
            # TODO
            if not self.is_valid_pair_key(key):
                raise KeyError(key)
            self.serialize_pair(key, val[key])

    def serialize_map(self, val):
        self.put('[')
        self.eol()
        # TODO "with self.indent..."
        self._indent_level += 1
        self.serialize_map_inner(val)
        self._indent_level -= 1
        self.put(']')
        self.eol()

    def serialize_any(self, data):
        if isinstance(data, collections.Mapping):
            self.serialize_map(data)
        elif isinstance(data, six.string_types):
            self.serialize_string(data)
        elif isinstance(data, int):
            self.serialize_int(data)
        elif isinstance(data, float):
            self.serialize_float(data)

    def serialize(self, data):
        # TODO
        self._indent_level = 0
        self._output = ''

        # TODO
        self.serialize_map_inner(data)
        return self._output



"""Parser utilities for the Able configuration format"""

from __future__ import print_function

import argparse
import json
import os
import pprint

import lictionary

from able import parser, serializer


class Semantics(parser.AbleSemantics):
    """Convert parsed items into the appropriate type."""

    # pylint: disable=missing-docstring,no-self-use

    def decimal_integer(self, ast):
        return int(''.join(ast))

    def hex_integer(self, ast):
        return int(''.join(ast), 16)

    def binary_integer(self, ast):
        return int(''.join(ast), 2)

    def float(self, ast):
        return float(''.join(ast))

    def pair(self, ast):
        key, value = ast
        return (key, value)

    def list(self, ast):
        return lictionary.Lictionary(*ast)

    def implicit_list(self, ast):
        return lictionary.Lictionary(*ast)


class JSONEncoder(json.JSONEncoder):
    """Specialize JSONEncoder to handle Lictionary."""

    def default(self, o):  # pylint: disable=method-hidden
        if isinstance(o, lictionary.Lictionary):
            return o.as_list()
        return JSONEncoder.default(self, o)


def read_grammar():
    """Read the contents of the Able grammar file"""
    script_path = os.path.dirname(os.path.realpath(__file__))
    grammar_path = os.path.join(script_path, 'grammar.ebnf')
    with open(grammar_path) as grammar:
        return grammar.read()


def parse(string, rule_name='start'):
    """Parse an Able string."""
    return parser.AbleParser().parse(
        string, rule_name=rule_name, semantics=Semantics())


def serialize(data):
    """Format |data| in Able."""
    return serializer.Serializer().serialize(data)


def parse_cli_args():
    """Parse command-line arguments"""
    arper = argparse.ArgumentParser(description='parse an Able file')
    arper.add_argument('path')
    arper.add_argument('-j', '--to-json', action='store_true')
    return arper.parse_args()


def main():
    """Entry point for 'python -m able'"""
    cli_args = parse_cli_args()
    with open(cli_args.path) as rfile:
        ast = parse(rfile.read())
    if cli_args.to_json:
        print(
            json.dumps(ast, cls=JSONEncoder, indent=2, separators=(',', ': ')))
    else:
        pprint.pprint(ast)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# CAVEAT UTILITOR
#
# This file was automatically generated by TatSu.
#
#    https://pypi.python.org/pypi/tatsu/
#
# Any changes you make to it will be overwritten the next time
# the file is generated.


from __future__ import print_function, division, absolute_import, unicode_literals

from tatsu.buffering import Buffer
from tatsu.parsing import Parser
from tatsu.parsing import tatsumasu
from tatsu.util import re, generic_main  # noqa


KEYWORDS = {}  # type: ignore


class AbleBuffer(Buffer):
    def __init__(
        self,
        text,
        whitespace=None,
        nameguard=None,
        comments_re=None,
        eol_comments_re='#([^\\n]*?)$',
        ignorecase=None,
        namechars='',
        **kwargs
    ):
        super(AbleBuffer, self).__init__(
            text,
            whitespace=whitespace,
            nameguard=nameguard,
            comments_re=comments_re,
            eol_comments_re=eol_comments_re,
            ignorecase=ignorecase,
            namechars=namechars,
            **kwargs
        )


class AbleParser(Parser):
    def __init__(
        self,
        whitespace=None,
        nameguard=None,
        comments_re=None,
        eol_comments_re='#([^\\n]*?)$',
        ignorecase=None,
        left_recursion=True,
        parseinfo=True,
        keywords=None,
        namechars='',
        buffer_class=AbleBuffer,
        **kwargs
    ):
        if keywords is None:
            keywords = KEYWORDS
        super(AbleParser, self).__init__(
            whitespace=whitespace,
            nameguard=nameguard,
            comments_re=comments_re,
            eol_comments_re=eol_comments_re,
            ignorecase=ignorecase,
            left_recursion=left_recursion,
            parseinfo=parseinfo,
            keywords=keywords,
            namechars=namechars,
            buffer_class=buffer_class,
            **kwargs
        )

    @tatsumasu()
    def _start_(self):  # noqa
        self._implicit_list_()
        self.name_last_node('@')
        self._check_eof()

    @tatsumasu()
    def _sign_(self):  # noqa
        with self._choice():
            with self._option():
                self._token('+')
            with self._option():
                self._token('-')
            self._error('no available options')

    @tatsumasu()
    def _DECIMAL_(self):  # noqa
        self._token('.')

    @tatsumasu()
    def _DECIMAL_FRAGMENT_(self):  # noqa
        self._pattern(r'[0-9]+')

    @tatsumasu()
    def _HEX_FRAGMENT_(self):  # noqa
        self._pattern(r'[0-9a-fA-F]+')

    @tatsumasu()
    def _BINARY_FRAGMENT_(self):  # noqa
        self._pattern(r'[01]+')

    @tatsumasu()
    def _hex_prefix_(self):  # noqa
        with self._choice():
            with self._option():
                self._token('0x')
            with self._option():
                self._token('0X')
            self._error('no available options')

    @tatsumasu()
    def _binary_prefix_(self):  # noqa
        with self._choice():
            with self._option():
                self._token('0b')
            with self._option():
                self._token('0B')
            self._error('no available options')

    @tatsumasu()
    def _numeric_float_(self):  # noqa
        with self._optional():
            self._sign_()
        self._DECIMAL_FRAGMENT_()
        self._DECIMAL_()
        self._DECIMAL_FRAGMENT_()

    @tatsumasu()
    def _infinity_(self):  # noqa
        self._pattern(r'(?i)[+-]?inf(inity)?')

    @tatsumasu()
    def _float_(self):  # noqa
        with self._choice():
            with self._option():
                self._numeric_float_()
            with self._option():
                self._token('nan')
            with self._option():
                self._infinity_()
            self._error('no available options')

    @tatsumasu()
    def _hex_integer_(self):  # noqa
        with self._optional():
            self._sign_()
        self._hex_prefix_()
        self._HEX_FRAGMENT_()

    @tatsumasu()
    def _binary_integer_(self):  # noqa
        with self._optional():
            self._sign_()
        self._binary_prefix_()
        self._BINARY_FRAGMENT_()

    @tatsumasu()
    def _decimal_integer_(self):  # noqa
        with self._optional():
            self._sign_()
        self._DECIMAL_FRAGMENT_()

    @tatsumasu()
    def _number_(self):  # noqa
        with self._choice():
            with self._option():
                self._float_()
            with self._option():
                self._hex_integer_()
            with self._option():
                self._binary_integer_()
            with self._option():
                self._decimal_integer_()
            self._error('no available options')

    @tatsumasu()
    def _double_quoted_string_(self):  # noqa
        self._token('"')
        self._pattern(r'[^"]+')
        self.name_last_node('@')
        self._token('"')

    @tatsumasu()
    def _single_quoted_string_(self):  # noqa
        self._token("'")
        self._pattern(r"[^']+")
        self.name_last_node('@')
        self._token("'")

    @tatsumasu()
    def _string_(self):  # noqa
        with self._choice():
            with self._option():
                self._double_quoted_string_()
            with self._option():
                self._single_quoted_string_()
            self._error('no available options')

    @tatsumasu()
    def _pair_(self):  # noqa
        self._pattern(r'[\w]+')
        self.name_last_node('@')
        self._token(':')
        self._value_()
        self.name_last_node('@')

    @tatsumasu()
    def _value_(self):  # noqa
        with self._choice():
            with self._option():
                self._number_()
            with self._option():
                self._string_()
            with self._option():
                self._pair_()
            with self._option():
                self._list_()
            self._error('no available options')

    @tatsumasu()
    def _list_(self):  # noqa
        self._token('[')

        def block1():
            self._value_()
        self._closure(block1)
        self.name_last_node('@')
        self._token(']')

    @tatsumasu()
    def _implicit_list_(self):  # noqa

        def block0():
            self._value_()
        self._closure(block0)


class AbleSemantics(object):
    def start(self, ast):  # noqa
        return ast

    def sign(self, ast):  # noqa
        return ast

    def DECIMAL(self, ast):  # noqa
        return ast

    def DECIMAL_FRAGMENT(self, ast):  # noqa
        return ast

    def HEX_FRAGMENT(self, ast):  # noqa
        return ast

    def BINARY_FRAGMENT(self, ast):  # noqa
        return ast

    def hex_prefix(self, ast):  # noqa
        return ast

    def binary_prefix(self, ast):  # noqa
        return ast

    def numeric_float(self, ast):  # noqa
        return ast

    def infinity(self, ast):  # noqa
        return ast

    def float(self, ast):  # noqa
        return ast

    def hex_integer(self, ast):  # noqa
        return ast

    def binary_integer(self, ast):  # noqa
        return ast

    def decimal_integer(self, ast):  # noqa
        return ast

    def number(self, ast):  # noqa
        return ast

    def double_quoted_string(self, ast):  # noqa
        return ast

    def single_quoted_string(self, ast):  # noqa
        return ast

    def string(self, ast):  # noqa
        return ast

    def pair(self, ast):  # noqa
        return ast

    def value(self, ast):  # noqa
        return ast

    def list(self, ast):  # noqa
        return ast

    def implicit_list(self, ast):  # noqa
        return ast


def main(filename, startrule, **kwargs):
    with open(filename) as f:
        text = f.read()
    parser = AbleParser()
    return parser.parse(text, startrule, filename=filename, **kwargs)


if __name__ == '__main__':
    import json
    from tatsu.util import asjson

    ast = generic_main(main, AbleParser, name='Able')
    print('AST:')
    print(ast)
    print()
    print('JSON:')
    print(json.dumps(asjson(ast), indent=2))
    print()



#!/usr/bin/env python

# Copyright 2017 Nicholas Bishop
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=missing-docstring

from setuptools import setup

setup(
    name='able',
    version='0.1.4',
    url='https://github.com/nicholasbishop/able',
    author='Nicholas Bishop',
    author_email='nicholasbishop@gmail.com',
    license='Apache 2.0',
    packages=['able'],
    install_requires=['TatSu~=4.2.3', 'lictionary~=0.1.2', 'six'],
    package_data={'able': ['grammar.ebnf']},
)
